pub const LEVEL_WIDTH: usize = 50;
pub const LEVEL_HEIGHT: usize = 25;

#[derive(Copy, Clone, Debug, PartialEq)]
pub enum LevelElement {
    Wall,
    Floor,
    StairDown,
    StairUp,
}

#[derive(Copy, Clone, Debug, PartialEq)]
pub struct Level {
    structure: [[LevelElement; LEVEL_HEIGHT]; LEVEL_WIDTH],
}

impl Level {
    pub fn new() -> Level {
        let mut s = [[LevelElement::Floor; LEVEL_HEIGHT]; LEVEL_WIDTH];
        for i in 0..LEVEL_WIDTH {
            s[i][0] = LevelElement::Wall;
            s[i][LEVEL_HEIGHT-1] = LevelElement::Wall;
        }
        for i in 0..LEVEL_HEIGHT {
            s[0][i] = LevelElement::Wall;
            s[LEVEL_WIDTH-1][i] = LevelElement::Wall;
        }
        s[10][10] = LevelElement::Wall;
        s[11][10] = LevelElement::Wall;
        s[10][11] = LevelElement::Wall;
        s[11][11] = LevelElement::Wall;
        s[1][20]  = LevelElement::Wall;
        s[20][1]  = LevelElement::Wall;
        s[20][LEVEL_HEIGHT-2]  = LevelElement::Wall;
        s[LEVEL_WIDTH-2][20] = LevelElement::Wall;
        s[4][4] = LevelElement::StairDown;
        s[45][20] = LevelElement::StairUp;
        Level {
            structure: s
        }
    }
    pub fn get_element(&self, x: i16, y: i16) -> Option<LevelElement> {
        if x < 0 || y < 0 {
            return None;
        }
        if x >= LEVEL_WIDTH as i16 || y >= LEVEL_HEIGHT as i16 {
            return None;
        }
        Some(self.structure[x as usize][y as usize])
    }
}

#[test]
fn test_get_element() {
    let l = Level::new();
    assert_eq!(l.get_element(0, 0).unwrap(), LevelElement::Wall);
    assert_eq!(l.get_element(4, 4).unwrap(), LevelElement::StairDown);
    assert_eq!(l.get_element(49, 24).unwrap(), LevelElement::Wall);
    assert_eq!(l.get_element(50, 25), None);
}
